#include "loadTracksWindow.h"
#include "ui_loadtracksWindow.h"

#include "windowMap.h"
#include "settings.h"

#include <QFileDialog>
#include <QColorDialog>
#include <QMessageBox>
#include <QCloseEvent>

LoadTracksWindow::LoadTracksWindow(WindowMap *parent, Settings* settings_p, Log *log) :
    QDialog(parent), log(log),windowMap(parent),settings(settings_p),
    ui(new Ui::LoadTracksWindow)
{
    ui->setupUi(this);
    currentDir.setPath(APPLICATION_PATH);

    setWindowTitle(tr("Load track"));
    QColor color = QColor(settings->getTrackToSniffColor());
    ui->colorLabel->setText(color.name());
    ui->colorLabel->setPalette(QPalette(color));
    ui->colorLabel->setAutoFillBackground(true);

    // Centering MapType items
    QAbstractItemView *viewMaps = ui->mapsType->view();
    QAbstractItemModel *modelMaps = viewMaps->model();

    for(int i=0; i < ui->mapsType->count();i++){
        modelMaps->setData(modelMaps->index(i, 0), QVariant(Qt::AlignCenter), Qt::TextAlignmentRole);
    }

    connect(ui->colorButton, SIGNAL(clicked()), this, SLOT(setColor()));
    connect(ui->browseButton, SIGNAL(clicked()), SLOT(browse()));
}

LoadTracksWindow::~LoadTracksWindow()
{
    delete ui;
}

void LoadTracksWindow::setColor(){


    QColor color = QColorDialog::getColor(QColor(settings->getTrackToSniffColor()), this);
        if (color.isValid())
        {
            ui->colorLabel->setText(color.name());
            ui->colorLabel->setPalette(QPalette(color));
            ui->colorLabel->setAutoFillBackground(true);
            settings->setTrackToSniffColor(color.name());
        }
}

void LoadTracksWindow::browse(){

    QString directory = QFileDialog::getOpenFileName(this,tr("Open XML Files"), currentDir.path(), tr("XML Files (*.gpx *.tcx)"));

    if (!directory.isNull())
    {
        currentDir.setPath(directory);

        if (!directory.isEmpty()) {
            if (ui->fileCombo->findText(directory) == -1)
                ui->fileCombo->addItem(directory);
            ui->fileCombo->setCurrentIndex(ui->fileCombo->findText(directory));
        }
    }

}


void LoadTracksWindow::closeEvent(QCloseEvent *event){
    //event->ignore();
    log->debug("CLOSE EVENT OF LOADTRACKS!!!!!!!!!!!!!!!!!!!");
    QMessageBox::warning(this, tr("GPSSniffer"),QString(tr("No file selected!")));
    this->hide();
}

void LoadTracksWindow::showEvent(QShowEvent *){

    if(windowMap->getMode()==Mode_NewActivityMode){
        // loading track in new activity
        ui->mapsType->setCurrentIndex(settings->getMapType());
        ui->mapsType->setEnabled(false);
    }
}

void LoadTracksWindow::accept(){


    QString fileName = ui->fileCombo->currentText();
    if(!fileName.endsWith(".gpx",Qt::CaseInsensitive) && !fileName.endsWith(".tcx",Qt::CaseInsensitive) ){
        QMessageBox::warning(this, tr("GPSSniffer"),QString(tr("No file selected!")));
    }else{
        Track* toSniff = readFromXML(fileName);
        log->debug("XML readed without problem");
        windowMap->setTrackToSniff(toSniff);

        if(windowMap->getMode()==Mode_LoadTracksWindow){
            settings->setMapType((MapType)ui->mapsType->currentIndex());
            windowMap->startNetworking();
            windowMap->startMaps();
        }else{

            if(windowMap->simulatingGPS()){
                windowMap->desactivatePossitionPaint();
            }
        }
        windowMap->updateSummary();
        windowMap->drawFullTrackMaps(toSniff);
        settings->setIsConfigured(true);
        windowMap->fixZoomButtons();
        this->hide();
    }


}

