#include "settingsWindow.h"
#include "windowMap.h"
#include "ui_settingsWindow.h"

#include <QAbstractItemView>
#include <QFileDialog>
#include <QColorDialog>
#include <QCloseEvent>

/*
  * populate settings widget class
  */

SettingsWindow::SettingsWindow(WindowMap *parent, Settings *sett, Log *log)
    : QDialog(parent),
    activitySettings(sett),
    activityMap(parent),
    log(log),
    ui(new Ui::SettingsWindow){

    ui->setupUi(this);

    currentDir = new QDir(APPLICATION_PATH);

    // Centering all TypeActivity items
    QAbstractItemView *viewType = ui->typeActivity->view();
    QAbstractItemModel *modelType = viewType->model();


    for(int i=0; i < ui->typeActivity->count();i++){

        modelType->setData(modelType->index(i, 0), QVariant(Qt::AlignCenter), Qt::TextAlignmentRole);
    }

    QColor color = QColor(activitySettings->getTrackColor());
    ui->colorLabel->setText(color.name());
    ui->colorLabel->setPalette(QPalette(color));
    ui->colorLabel->setAutoFillBackground(true);

    // Centering GPS interval items
    QAbstractItemView *viewGPS = ui->gpsInterval->view();
    QAbstractItemModel *modelGPS = viewGPS->model();

    for(int i=0; i < ui->gpsInterval->count();i++){
        modelGPS->setData(modelGPS->index(i, 0), QVariant(Qt::AlignCenter), Qt::TextAlignmentRole);
    }

    // Centering MapType items
    QAbstractItemView *viewMaps = ui->mapsType->view();
    QAbstractItemModel *modelMaps = viewMaps->model();

    for(int i=0; i < ui->mapsType->count();i++){
        modelMaps->setData(modelMaps->index(i, 0), QVariant(Qt::AlignCenter), Qt::TextAlignmentRole);
    }

    currentDir->setPath(APPLICATION_PATH);

    connect(ui->browseButton, SIGNAL(clicked()), SLOT(browse()));
    connect(ui->colorButton, SIGNAL(clicked()), this, SLOT(setColor()));

    connect(ui->typeActivity,SIGNAL(currentIndexChanged(int)),this,SLOT(changeEditText()));
    connect(ui->tcxButton,SIGNAL(toggled(bool)),this,SLOT(changeTCXTypeFile(bool)));

    // if simulating mode connect with gpssimulate



}

SettingsWindow::~SettingsWindow()
{
    delete ui;
}

void SettingsWindow::changeEditText(){
    ui->fileCombo->setEditText(QString(APPLICATION_PATH).append(QDateTime::currentDateTime().toString(CONTINOUS_DATE_FORMAT))
                               .append("_").append(ui->typeActivity->currentText()).append(getExtension()));
}

void SettingsWindow::changeTCXTypeFile(bool checked){

    QString extension;
    if(checked)
        extension=QString(".tcx");
    else
        extension=QString(".gpx");

    ui->fileCombo->setEditText(QString(APPLICATION_PATH).append(QDateTime::currentDateTime().toString(CONTINOUS_DATE_FORMAT))
                               .append("_").append(ui->typeActivity->currentText()).append(extension));

}

QString SettingsWindow::getExtension(){

    if(ui->tcxButton->isChecked()){
        return QString(".tcx");
    }else{
        return QString(".gpx");
    }
}

void SettingsWindow::showEvent(QShowEvent *){

    if(activityMap->simulatingGPS()){
        setWindowTitle(tr("Simulating GPS Activity"));
        ui->xmlTypeLabel->setVisible(false);
        ui->gpxButton->setVisible(false);
        ui->tcxButton->setVisible(false);
        //ui->xmlTypeLayout->setVisible(false);

    }else{
        setWindowTitle(tr("GPS Activity"));
        ui->xmlTypeLabel->setVisible(true);
        //ui->xmlTypeLayout->setVisible(true);
        ui->gpxButton->setVisible(true);
        ui->tcxButton->setVisible(true);
        ui->fileCombo->setEnabled(true);
        ui->browseButton->setEnabled(true);
        ui->fileCombo->setEditText(QString(APPLICATION_PATH).append(QDateTime::currentDateTime().toString(CONTINOUS_DATE_FORMAT))
                                   .append("_").append(ui->typeActivity->currentText()).append(getExtension()));
    }

    //if(activitySettings->getIsConfigured() && !activityMap->simulatingGPS()){
     if(activitySettings->getIsConfigured()){
        //log->debug("DISABLING BUTTONS...");
        ui->browseButton->setEnabled(false);
        ui->fileCombo->setEnabled(false);
        ui->gpxButton->setEnabled(false);
        ui->tcxButton->setEnabled(false);
    }
}

void SettingsWindow::browse(){

    //log->debug("browse slot...");
    //log->debug("current dir:" + currentDir->path());

    QString directory;

    if(activityMap->simulatingGPS()){
        //log->debug("simulating GPS...");
        directory = QFileDialog::getOpenFileName(this,tr("Open XML Files"), currentDir->path(), tr("XML Files (*.gpx *.tcx)"));
    }else{
        directory = QFileDialog::getExistingDirectory(this, tr("Open Directory"),currentDir->path(),
                                                      QFileDialog::ShowDirsOnly
                                                    | QFileDialog::DontResolveSymlinks);
    }

    //QString directory = QFileDialog::getExistingDirectory (this, tr("Directory"), currentDir->path());
    //QString directory = QFileDialog::getExistingDirectory (this, tr("Directory"), "./");


    log->debug("directory:" + directory);
    if ( directory.isNull() == false )
    {
        currentDir->setPath(directory);

        log->debug("going to catch file...");

        if (!directory.isEmpty()) {
            if (ui->fileCombo->findText(directory) == -1)
                ui->fileCombo->addItem(directory);
            ui->fileCombo->setCurrentIndex(ui->fileCombo->findText(directory));
        }
    }

}

void SettingsWindow::setColor(){


    QColor color = QColorDialog::getColor(QColor(activitySettings->getTrackColor()), this);
        if (color.isValid())
        {
            ui->colorLabel->setText(color.name());
            ui->colorLabel->setPalette(QPalette(color));
            ui->colorLabel->setAutoFillBackground(true);
            activitySettings->setTrackColor(color.name());
        }
}


void SettingsWindow::closeEvent(QCloseEvent *event){
    if(!activitySettings->getIsConfigured()){
        this->parentWidget()->hide();
        this->parentWidget()->parentWidget()->show();
        this->hide();
    }else{

        if(activityMap->simulatingGPS()&&!activityMap->hasTrackToSimulate()){
            event->ignore();
        }else{
            this->hide();
        }
    }
}



void SettingsWindow::accept(){
    this->hide();

    log->debug("--- SettingsWindows --- closeEvent ");

    if(!activitySettings->getIsConfigured()){

        // first time configuration

        activitySettings->setActivity(ui->typeActivity->currentText());
        activitySettings->setGpsInterval(intervalFromCombo(ui->gpsInterval->currentIndex()));

        log->debug(QString("---- ONLINE:%1").arg(ui->onlineButton->isChecked()));
        activitySettings->setIsOnline(ui->onlineButton->isChecked());

        activitySettings->setXmlFile(ui->fileCombo->currentText());

        activitySettings->setMapType((MapType)ui->mapsType->currentIndex());


        log->debug("going to add activity:" + activitySettings->getActivity());

        activityMap->setTrack(activitySettings->getXmlFile(),activitySettings->getActivity());

        activitySettings->setIsConfigured(true);
        log->debug("starting services");
        activityMap->startServices();
        log->debug("done");

        //activityMap->startSPositioningTimer();

        if(activityMap->simulatingGPS()){
            log->debug(QString("going to read from XML, file:%1").arg(activitySettings->getXmlFile()));
            Track* toDo = readFromXML(activitySettings->getXmlFile());
            log->debug("XML readed without problem");
            activityMap->setTrackToSimulate(toDo->getGpsPoints());
            activityMap->changeUpdateInterval(activitySettings->getGpsInterval());

            emit(activityMap->simulatedPossitionUpdated());
            activityMap->startSimulatePossitioning();

        }else{
            activityMap->startAutosaveTimer(60000*TIME_MIN_TIMEAUTOSAVE);

        }
        log->debug("accept done");

    }else{
        //need to see what has changed
         log->debug("going to configure some parameters!");

        if(activitySettings->getActivity()!=QString(ui->typeActivity->currentText())){
            activityMap->getTrack()->addActivity(activitySettings->getActivity());
            log->debug("activity changed to " + activitySettings->getActivity());
        }
        if(activitySettings->getGpsInterval()!=intervalFromCombo(ui->gpsInterval->currentIndex())){
            log->debug("interval changed, going to change it from location");
            activitySettings->setGpsInterval(intervalFromCombo(ui->gpsInterval->currentIndex()));
            activityMap->changeUpdateInterval(activitySettings->getGpsInterval());
            log->debug(QString("Interval setted to: %1").arg(activitySettings->getGpsInterval()));
        }
        if(activitySettings->getIsOnline()!=ui->onlineButton->isChecked()){
            activitySettings->setIsOnline(ui->onlineButton->isChecked());
            if(!activitySettings->getIsOnline()){
                log->debug("SettingsWindow stopNetworking");
                activityMap->stopNetworking();
            }if(activitySettings->getIsOnline()){
                log->debug("SettingsWindow startNetworking");
                activityMap->startNetworking();
            }
        }

        if(activitySettings->getMapType()!=(MapType)ui->mapsType->currentIndex()){
            activitySettings->setMapType((MapType)ui->mapsType->currentIndex());
            activityMap->changeMapType(activitySettings->getMapType());
            log->debug(QString("Changing maps to: %1").arg(activitySettings->getMapType()));
            activityMap->clearAllMaps();
        }

    }
    //activityMap->setCenter(FIRST_LATITUDE,FIRST_LONGITUDE);

}

void SettingsWindow::setActivityType(){

}

int SettingsWindow::intervalFromCombo(int index){

    log->debug(QString("index from combo:%1").arg(index));
    int returnValue=2000;

    switch(index){

    case 1:
        returnValue=5000; //5 seconds
        break;
    case 2:
        returnValue=10000; //10 seconds
        break;
    case 3:
        returnValue=20000;  //20 seconds
        break;
    case 4:
        returnValue=30000;  //20 seconds
        break;
    case 5:
        returnValue=60000;
        break;
    default:
    case 0:
        returnValue=2000; // 2 seconds
        break;
    }
    return returnValue;
}

