#include "track.h"
#include <QDebug>
#include <QXmlStreamWriter>
#include <QFile>

#include <QtCore/qmath.h>
#include "utils.h"


Summarized::Summarized(){

    startTime=QDateTime::currentDateTime();

    numPoints=0;

    duration=0.0;
    distance=0.0;
    avgSpeed=0.0;
    maxSpeed=0.0;
    elevationGain=0.0;
    elevationLoss=0.0;
    minElevation=0.0;
    maxElevation=0.0;
    avgPace=0.0;
    bestPace=0.0;
}

QString Summarized::toSumString(){
    return QString("Summarized={numPoints: %1, startTime: %2, duration: %3, distance: %4, avgSpeed: %5, maxSpeed: %6, elevationGain: %7, elevationLoss: %8, minElevation: %9, maxElevation: %10, avgPace: %11, minPace: %12}")
            .arg(numPoints).arg(startTime.toString(CLEAN_DATE_FORMAT)).arg(duration).arg(distance)
            .arg(avgSpeed).arg(maxSpeed).arg(elevationGain).arg(elevationLoss).arg(minElevation).arg(maxElevation).arg(avgPace).arg(bestPace);
}
void Summarized::addSummary(Summarized summary){

    duration+=summary.getDuration();
    distance=distance+summary.getDistance();
    // avgSpeed of summary is not Avg, is speed of last point!
    if(summary.getAvgPace()!=-1)
        avgSpeed=((avgSpeed*numPoints)+summary.getAvgSpeed())/(numPoints+1);

    elevationGain+=summary.getElevationGain();
    elevationLoss+=summary.getElevationLoss();

    if(minElevation!=0 && summary.minElevation!=0){
        minElevation=min(minElevation,summary.minElevation);
    }else if(minElevation==0 && summary.minElevation!=0){
        minElevation=summary.minElevation;
    }

    maxElevation=max(maxElevation,summary.maxElevation);
    maxSpeed=max(maxSpeed,summary.maxSpeed);

    if(bestPace!=0 && summary.avgPace!=0){
        bestPace=min(bestPace,summary.avgPace);
    }else if(bestPace==0 && summary.avgPace!=0){
        bestPace=summary.avgPace;
    }

    avgPace=(1000*duration)/(60*distance);
    numPoints++;
}

Summarized& Summarized::account(GpsPoint point){

    // this just is usefull for accounting the diference
    // between two points. The summary MUST be new!!!
    // data in here is not secure and not controlled.

    numPoints++;
    startTime=point.getTime();
    endTime=point.getTime();
    maxSpeed=point.getSpeed();
    avgSpeed=point.getSpeed();
    minElevation=point.getElevation();
    maxElevation=point.getElevation();

    return (*this);
}

Summarized& Summarized::account(GpsPoint first, GpsPoint last){

    // this just is usefull for accounting the diference
    // between two points. The summary MUST be new!!!
    // data in here is not secure and not controlled.

    numPoints++;
    distance=first.distance(last);
    duration=first.getTime().secsTo(last.getTime());
    startTime=first.getTime();
    endTime=last.getTime();
    double lastEle=last.getElevation();
    double firstEle=first.getElevation();
    if(lastEle>=firstEle)
        elevationGain=lastEle-firstEle;
    else
        elevationLoss=firstEle-lastEle;

    maxSpeed=max(first.getSpeed(),last.getSpeed());
    // keep speed of last point, will be usefull
    // in add function
    avgSpeed=last.getSpeed();
    minElevation=min(first.getElevation(),last.getElevation());
    maxElevation=max(first.getElevation(),last.getElevation());
    avgPace=(1000*duration)/(60*distance);

    return (*this);
}


GpsPoint::GpsPoint(double latitude, double longitude, int elevation)
    :latitude(latitude),
    longitude(longitude),
      elevation(elevation),speed(0),direction(0),magneticVariation(0),horizontalAccuracy(0),verticalAccuracy(0){
}

GpsPoint::GpsPoint(double latitude, double longitude, int elevation,QDateTime time,
                   qreal speed, qreal direction, qreal magneticVariation, qreal horizontalAccuracy, qreal verticalAccuracy)
:latitude(latitude),longitude(longitude),elevation(elevation),time(time),
  speed(speed), direction(direction), magneticVariation(magneticVariation),
  horizontalAccuracy(horizontalAccuracy),verticalAccuracy(verticalAccuracy){

}

GpsPoint::GpsPoint(double latitude, double longitude, int elevation,QDateTime time,
                   qreal speed, qreal direction, qreal magneticVariation, qreal horizontalAccuracy, qreal verticalAccuracy,
                   qreal distancePrev, QTime timeToPrev)
:latitude(latitude),longitude(longitude),elevation(elevation),time(time),
  speed(speed), direction(direction), magneticVariation(magneticVariation),
  horizontalAccuracy(horizontalAccuracy),verticalAccuracy(verticalAccuracy),distancePrev(distancePrev), timeToPrev(timeToPrev){

}

QString GpsPoint::toString(){
    QString point = QString("GpsPoint{latitude: %1, longitude: %2, elevation: %3, time: %4, speed: %5, direction: %6, magneticVariation: %7, horizontalAccuracy: %8, verticalAccuracy: %9:, distancePrev: %10}")
            .arg(latitude).arg(longitude).arg(elevation).arg(time.toString("dd-MM-yyyy hh:mm:ss")).arg(speed).arg(direction).arg(magneticVariation)
            .arg(horizontalAccuracy).arg(verticalAccuracy).arg(QString::number(distancePrev,'f',10));
    return point;
}

GpsPoint& GpsPoint::operator= (const GpsPoint &p){


    latitude=p.latitude;
    longitude=p.longitude;
    elevation=p.elevation;
    speed=p.speed;
    direction=p.direction;
    magneticVariation=p.magneticVariation;
    time=p.time;
    distancePrev=p.distancePrev;
    timeToPrev=p.timeToPrev;
    horizontalAccuracy=p.horizontalAccuracy;
    verticalAccuracy=p.verticalAccuracy;

    return(*this);
}



double GpsPoint::distance(GpsPoint p){

    double latitude1 = toRad(latitude);
    double longitude1 = toRad(longitude);

    double latitude2 = toRad(p.latitude);
    double longitude2 = toRad(p.longitude);

    double temp = sin(latitude1)*sin(latitude2)+cos(latitude1)*cos(latitude2)*cos(longitude2-longitude1);

    double dist= acos(temp)*EARTH_RADIUS*1000;
    //qDebug() << "distance between points: " << QString::number(dist,'f',10);
    return dist;
}

int operator== (const GpsPoint& a, const GpsPoint& b)
{
if (a.latitude==b.latitude&&a.longitude==b.longitude&&a.elevation==b.elevation&&a.speed==b.speed
        &&a.direction==b.direction &&a.magneticVariation==b.magneticVariation&&a.time==b.time
        &&a.horizontalAccuracy==b.horizontalAccuracy&&a.verticalAccuracy==b.verticalAccuracy
        &&a.distancePrev==b.distancePrev&&a.timeToPrev==b.timeToPrev)
    return 1;
else
    return 0;
}

int operator!= (const GpsPoint& a, const GpsPoint& b)
{
if (a.latitude!=b.latitude||a.longitude!=b.longitude||a.elevation!=b.elevation||a.speed!=b.speed
        ||a.direction!=b.direction || a.magneticVariation!=b.magneticVariation||a.time!=b.time
        ||a.horizontalAccuracy!=b.horizontalAccuracy||a.verticalAccuracy!=b.verticalAccuracy
        ||a.distancePrev!=b.distancePrev||a.timeToPrev!=b.timeToPrev)
    return 1;
else
    return 0;
}



Lap::Lap(int number)
    :number(number){
    numPoints=0;

}

Lap::~Lap(){
    for (int i = 0; i < points.size(); ++i) {
        delete points.at(i);
    }
}


Summarized* Lap::addPoint(GpsPoint *point){
    //qDebug() <<"--- Lap::addPoint ---";
    Summarized *summary = new Summarized();
    if(points.size()>0){
        GpsPoint* last = points.last();
        point->setDistancePrev(point->distance(*last));
        //qDebug() << "DistancePrev:" << QString::number(point->getDistancePrev(),'f',10);
        summary->account(*last,*point);
    }else{
        //qDebug() << "First lap point!";
        summary->account(*point);
    }
    addSummary(*summary);
    //qDebug() << "Track summary:" << toSumString();

    points.append(point);
    return summary;
}

QString Lap::toString(){
    QString lap = QString("Lap={number:").append(QString::number(number)).append(", numPoints:").append(QString::number(numPoints));

    QListIterator<GpsPoint*> i(points);
    while (i.hasNext()) {
       GpsPoint* p = i.next();
       lap.append(QString(", %1").arg(p->toString()));
    }
    lap.append(QString("}"));
    return lap;
}

Activity::Activity(QString sport)
    :sport(sport){
    id=QDateTime::currentDateTime();
    numLaps=0;
    addLap();
}

Activity::~Activity(){
    for (int i = 0; i < laps.size(); ++i) {
        delete laps.at(i);
    }
}

QString Activity::toString(){
    QString activity = QString("Activity={sport:%1, id:%2, numLaps:%3").arg(sport).arg(id.toString(CLEAN_DATE_FORMAT)).arg(numLaps);

    QListIterator<Lap*> i(laps);
    while(i.hasNext()){
        Lap* l=i.next();
        activity.append(QString(", %1").arg(l->toString()));
    }
    activity.append(QString("}"));
    return activity;
}

void Activity::addLap(){
    numLaps++;
    Lap* newLap = new Lap(numLaps);
    laps.append(newLap);
}

Summarized* Activity::addPoint(GpsPoint* point){

    //qDebug() <<"--- Activity::addPoint ---";
    Lap* lastLap = laps.last();
    Summarized* summary = lastLap->addPoint(point);
    addSummary(*summary);
    //qDebug() << "Track summary:" << toSumString();

    return summary;

}


Track::Track(QString file,QString nameTrack)
    :fileName(file),name(nameTrack){
    numActivities=0;
    addActivity(nameTrack);
}

Track::Track()
    :fileName("noname"),name("noname"){
    numActivities=0;
    addActivity(name);
}

Track::~Track(){
    for (int i = 0; i < activities.size(); ++i) {
        delete activities.at(i);
    }
}

void Track::addLap(){

    if(activities.size()>0){
        Activity* activity = activities.last();
        activity->addLap();
    }
}


void Track::addActivity(QString sport){

    name=sport;
    Activity* activity = new Activity(sport);
    activities.append(activity);
    numActivities++;
}

Summarized*  Track::addPoint(GpsPoint* point){

    //qDebug() <<"--- Track::addPoint ---";
    //qDebug() << "Adding point: " << point->toString();

    Activity* actualActivity = activities.last();
    Summarized* summary = actualActivity->addPoint(point);
    addSummary(*summary);
    //qDebug() << "Track summary:" << toSumString();

    return summary;
}

QString Track::toString(){

    QString track = QString("Track={fileName:%2, numActivities:%3").arg(fileName).arg(numActivities);

    QListIterator<Activity*> i(activities);
    while(i.hasNext()){
        Activity* a = i.next();
        track.append(QString(", %1").arg(a->toString()));
    }
    track.append(QString("}"));
    return track;
}

 QList<GpsPoint*> Track::getGpsPoints(){

     QList<GpsPoint*> points=QList<GpsPoint*>();

     if(numPoints==0)
         return points;

     QListIterator<Activity*> iAct(activities);
     while (iAct.hasNext()) {
         Activity* act = iAct.next();

         QListIterator<Lap*> iLap(act->getLaps());
         while (iLap.hasNext()) {
             Lap* lap = iLap.next();

             QListIterator<GpsPoint*> iPts(lap->getPoints());
             while (iPts.hasNext()) {
                 GpsPoint* point = iPts.next();
                 points.append(point);
             }
         }
     }
     return points;
 }






bool Track::saveToXML(){

    XMLFileType fileType;
    QFile file(fileName);

    if(file.exists())
        file.remove();

    if (!file.open(QIODevice::WriteOnly|QIODevice::Truncate)) {
        qDebug() << "Error: Cannot write file "
                 << qPrintable(fileName) << ": "
                 << qPrintable(file.errorString());
        return false;
    }

    if(fileName.endsWith(".tcx",Qt::CaseInsensitive)){
        fileType=XMLFile_TCX;
    }else if(fileName.endsWith(".gpx",Qt::CaseInsensitive)){
        fileType=XMLFile_GPX;
    }else{
        qDebug()<<"Error: XML not recognized. " << "nameFile:" << fileName;
    }

    QXmlStreamWriter xmlWriter(&file);

    xmlWriter.setAutoFormatting(true);

    xmlWriter.writeStartDocument();

    if(fileType==XMLFile_TCX){
        xmlWriter.writeStartElement("TrainingCenterDatabase");
        xmlWriter.writeAttribute("xmlns", TCX_XMLNS);
        xmlWriter.writeAttribute("xmlns:xsi", TCX_XMLNS_XSI);
        xmlWriter.writeAttribute("xsi:schemaLocation", TCX_XSI_SCHEMALOCATION);

        writeTCXCourses(&xmlWriter);
    }else{

        xmlWriter.writeStartElement("gpx");
        xmlWriter.writeAttribute("version",GPX_VERSION);
        xmlWriter.writeAttribute("creator",GPX_CREATOR);
        xmlWriter.writeAttribute("xmlns",GPX_XMLNS);

        writeGPXTracks(&xmlWriter);
    }




    //xmlWriter->writeEndElement();
    xmlWriter.writeEndDocument();
    file.close();
    qDebug() << "file closed";
    if (file.error()) {
        qDebug() << "Error on close: Cannot write file "
                 << qPrintable(fileName) << ": "
                 << qPrintable(file.errorString());
        return false;
    }
    return true;
}

void Track::writeGPXTracks(QXmlStreamWriter* xmlWriter){

    QListIterator<Activity*> i(activities);

    while (i.hasNext()) {
        Activity* act = i.next();
        //xmlWriter->writeStartElement("NextSport");
        writeGPXTrack(xmlWriter, act);
        xmlWriter->writeEndElement();
    }
    xmlWriter->writeEndElement();
    xmlWriter->writeEndElement();
}


void Track::writeTCXCourses(QXmlStreamWriter* xmlWriter){

    xmlWriter->writeStartElement("Courses");
    //xmlWriter->writeStartElement("MultiSportSession");

    //xmlWriter->writeTextElement("Id",startTime.toString(XML_DATE_FORMAT));

    QListIterator<Activity*> i(activities);

    while (i.hasNext()) {
        Activity* act = i.next();
        //xmlWriter->writeStartElement("NextSport");
        writeTCXCourse(xmlWriter, act);
        xmlWriter->writeEndElement();
    }
    xmlWriter->writeEndElement();
    xmlWriter->writeEndElement();
}

void Track::writeGPXTrack(QXmlStreamWriter* xmlWriter, Activity* act){

    xmlWriter->writeStartElement("trk");

    xmlWriter->writeTextElement("Name",act->getSport());

    QListIterator<Lap*> i(act->getLaps());
    while (i.hasNext()) {
        Lap* lap = i.next();
        writeGPXTrkseg(xmlWriter, lap);
    }
    xmlWriter->writeEndElement();


}

void Track::writeTCXCourse(QXmlStreamWriter* xmlWriter, Activity* act){


    xmlWriter->writeStartElement("Course");
    //xmlWriter->writeAttribute("Sport", act->getSport());
    xmlWriter->writeTextElement("Name",act->getSport());

    QListIterator<Lap*> i(act->getLaps());
    while (i.hasNext()) {
        Lap* lap = i.next();
        writeTCXLap(xmlWriter, lap);
    }
    xmlWriter->writeEndElement();
}

void Track::writeGPXTrkseg(QXmlStreamWriter* xmlWriter, Lap* lap){

    xmlWriter->writeStartElement("trkseg");

    QListIterator<GpsPoint*> i(lap->getPoints());
    while (i.hasNext()) {
        GpsPoint* point = i.next();
        writeGPXPoint(xmlWriter,point);
    }
    xmlWriter->writeEndElement();
}

void Track::writeTCXLap(QXmlStreamWriter* xmlWriter, Lap* lap){

    xmlWriter->writeStartElement("Lap");
    xmlWriter->writeAttribute("StartTime", lap->getStartTime().toString(XML_DATE_FORMAT));
    xmlWriter->writeTextElement("TotalTimeSeconds",QString::number(lap->getDuration()));
    xmlWriter->writeTextElement("DistanceMeters",QString::number(lap->getDistance(),'f',10));
    xmlWriter->writeTextElement("MaximumSpeed",QString::number(lap->getMaxSpeed()));

    QListIterator<GpsPoint*> i(lap->getPoints());
    xmlWriter->writeStartElement("Track");
    while (i.hasNext()) {
        GpsPoint* point = i.next();
        writeTCXPoint(xmlWriter,point);
    }
    xmlWriter->writeEndElement();
    xmlWriter->writeEndElement();
}

void Track::writeGPXPoint(QXmlStreamWriter* xmlWriter, GpsPoint* point){

    xmlWriter->writeStartElement("trkpt");
    xmlWriter->writeAttribute("lat",QString::number(point->getLatitude(),'f',20));
    xmlWriter->writeAttribute("lon",QString::number(point->getLatitude(),'f',20));


    xmlWriter->writeTextElement("ele",QString::number(point->getElevation(),'f',20));
    xmlWriter->writeTextElement("time",point->getTime().toString(XML_DATE_FORMAT));

    xmlWriter->writeEndElement();
}

void Track::writeTCXPoint(QXmlStreamWriter* xmlWriter, GpsPoint* point){

    xmlWriter->writeStartElement("Trackpoint");
    xmlWriter->writeTextElement("Time",point->getTime().toString(XML_DATE_FORMAT));
    xmlWriter->writeStartElement("Position");
        xmlWriter->writeTextElement("LatitudeDegrees",QString::number(point->getLatitude(),'f',20));
        xmlWriter->writeTextElement("LongitudeDegrees",QString::number(point->getLongitude(),'f',20));
    xmlWriter->writeEndElement();
    xmlWriter->writeTextElement("AltitudeMeters",QString::number(point->getElevation(),'f',20));
    xmlWriter->writeTextElement("DistanceMeters",QString::number(point->getDistancePrev(),'f',20));
    xmlWriter->writeEndElement();
}

