#ifndef TRACK_H
#define TRACK_H

#include <QList>
#include <QDateTime>
#include <QTime>
#include "constants.h"

class QXmlStreamWriter;
class GpsPoint;
class QXmlStreamReader;


class Summarized{

protected:

    int numPoints;
    double minLatitude;
    double maxLatitude;
    double minLongitude;
    double maxLongitude;
    int gpsInterval;
    QDateTime startTime;
    QDateTime endTime;
    double duration;
    double distance;
    qreal avgSpeed;
    qreal maxSpeed;
    qreal elevationGain;
    qreal elevationLoss;
    qreal minElevation;
    qreal maxElevation;
    qreal avgPace;
    qreal bestPace;

public:

    explicit Summarized();
    //virtual ~Summarized();

    /*
    inline double getDuration(){return duration;}
    inline double getDistance(){return distance;}
    inline qreal getAvgSpeed(){return avgSpeed;}
    inline qreal getMaxSpeed(){return maxSpeed;}
    inline qreal getElevationGain(){return elevationGain;}
    inline qreal getElevationLoss(){return elevationLoss;}
    inline qreal getMinElevation(){return minElevation;}
    inline qreal getMaxElevation(){return maxelevation;}
    inline qreal getAvgPace(){return avgPace;}
    inline qreal getBestPace(){return bestPace;}
    */

    void addSummary(Summarized summary);
    Summarized& account(GpsPoint point);
    Summarized& account(GpsPoint first, GpsPoint last);

    inline int getNumPoints(){return numPoints;}
    inline void setNumPoints(int p){numPoints=p;}

    inline double getMinLatitude(){return minLatitude;}
    inline void setMinLatitude(double latitude){minLatitude=latitude;}

    inline double getMaxLatitude(){return maxLatitude;}
    inline void setMaxLatitude(double latitude){maxLatitude=latitude;}

    inline double getMinLongitude(){return minLongitude;}
    inline void setMinLongitude(double longitude){minLongitude=longitude;}

    inline double getMaxLongitude(){return maxLongitude;}
    inline void setMaxLongitude(double longitude){maxLongitude=longitude;}

    inline int getGpsInterval(){return gpsInterval;}
    inline void setGpsInterval(int interval){gpsInterval=interval;}

    inline QDateTime getStartTime(){return startTime;}
    inline void setStartTime(QDateTime time){startTime=time;}

    inline QDateTime getEndTime(){return endTime;}
    inline void setEndTime(QDateTime time){endTime=time;}

    inline double getDuration(){return duration;}
    inline void setDuration(double duration_p){duration=duration_p;}

    inline double getDistance(){return distance;}
    inline void setDistance(double distance_p){distance=distance_p;}

    inline int getAvgSpeed(){return avgSpeed;}
    inline void setAvgSpeed(int speed){avgSpeed=speed;}

    inline int getMaxSpeed(){return maxSpeed;}
    inline void setMaxSpeed(int speed){maxSpeed=speed;}

    inline int getElevationGain(){return elevationGain;}
    inline void setElevationGain(int elevation){elevationGain=elevation;}

    inline int getElevationLoss(){return elevationLoss;}
    inline void setElevationLoss(int elevation){elevationLoss=elevation;}

    inline int getMinElevation(){return minElevation;}
    inline void setMinElevation(int elevation){minElevation=elevation;}

    inline int getMaxElevation(){return maxElevation;}
    inline void setMaxElevation(int elevation){maxElevation=elevation;}

    inline int getAvgPace(){return avgPace;}
    inline void setAvgPace(int time){avgPace=time;}

    inline int getBestPace(){return bestPace;}
    inline void setBestPace(int time){bestPace=time;}
    QString toSumString();
};


class GpsPoint{

private:
    double latitude;
    double longitude;
    double elevation;

    QDateTime time;
    qreal speed;
    qreal direction;
    qreal magneticVariation;
    qreal horizontalAccuracy;
    qreal verticalAccuracy;
    qreal distancePrev;
    QTime timeToPrev;
public:
    explicit GpsPoint(double latitude, double longitude, int elevation);
    explicit GpsPoint(double latitude, double longitude, int elevation,QDateTime time, qreal speed,
                      qreal direction, qreal magneticVariation,qreal horizontalAccuracy, qreal verticalAccuracy);
    explicit GpsPoint(double latitude, double longitude, int elevation,QDateTime time, qreal speed,
                      qreal direction, qreal magneticVariation,qreal horizontalAccuracy, qreal verticalAccuracy, qreal distancePrev, QTime timeToPrev);

    //virtual ~GpsPoint();

    inline double getLatitude(){return latitude;}
    inline void setLatitude(double lat){latitude=lat;}

    inline double getLongitude(){return longitude;}
    inline void setLongitude(double longitude_p){longitude=longitude_p;}

    inline double getElevation(){return elevation;}
    inline void setElevation(double elevation_p){elevation=elevation_p;}

    inline QDateTime getTime(){return time;}
    inline void setTime(QDateTime time_p){time=time_p;}

    inline qreal getSpeed(){return speed;}
    inline void setSpeed(qreal speed_p){speed=speed_p;}

    inline qreal getDirection(){return direction;}
    inline void setDirection(qreal direction_p){direction=direction_p;}

    inline qreal getMagneticVariation(){return magneticVariation;}
    inline void setMagneticVariation(qreal magneticVariation_p){magneticVariation=magneticVariation_p;}

    inline qreal getHorizontalAccuracy(){return horizontalAccuracy;}
    inline void setHorizontalAccuracy(qreal horizontalAccuracy_p){horizontalAccuracy=horizontalAccuracy_p;}

    inline qreal getVerticalAccuracy(){return verticalAccuracy;}
    inline void setVerticalAccuracy(qreal verticalAccuracy_p){verticalAccuracy=verticalAccuracy_p;}

    inline double getDistancePrev(){return distancePrev;}
    inline void setDistancePrev(double distance){distancePrev=distance;}

    QString toString();

    GpsPoint& operator= (const GpsPoint&);

    friend int operator== (const GpsPoint&, const GpsPoint&);
    friend int operator!= (const GpsPoint&, const GpsPoint&);
    friend QDataStream& operator<< (QDataStream&, const GpsPoint&);

    double distance(GpsPoint p);

};

class Lap: public Summarized{

private:
    int number;
    QList<GpsPoint*> points;

public:

    explicit Lap(int number);
    virtual ~Lap();

    inline int getNumber(){return number;}
    inline void setNumber(int number_p){number=number_p;}

    Summarized* addPoint(GpsPoint* point);
    inline QList<GpsPoint*> getPoints(){return points;}
    QString toString();

};

class Activity: public Summarized{

private:
    QString sport;
    int numLaps;
    QDateTime id;
    QList<Lap*> laps;

public:

    explicit Activity(QString sport);
    virtual ~Activity();

    inline QDateTime getId(){return id;}
    inline void setId(QDateTime time){id=time;}

    inline QString getSport(){return sport;}
    inline void setSport(QString sport_p){sport=sport_p;}

    inline int getNumLaps(){return numLaps;}
    inline void setNumLaps(int laps){numLaps=laps;}

    void addLap();
    Summarized* addPoint(GpsPoint* point);

    inline QList<Lap*> getLaps(){return laps;}
    QString toString();
};


class Track:public Summarized
{

private:

    QString fileName;
    QString name;
    int numActivities;
    QList<Activity*> activities;

    double minLat;
    double minLong;
    double maxLat;
    double maxLong;


    void writeTCXCourses(QXmlStreamWriter* xmlWriter);
    void writeTCXCourse(QXmlStreamWriter* xmlWriter, Activity* act);
    void writeTCXLap(QXmlStreamWriter* xmlWriter, Lap* lap);
    void writeTCXPoint(QXmlStreamWriter* xmlWriter, GpsPoint* point);
    void writeGPXTracks(QXmlStreamWriter* xmlWriter);
    void writeGPXTrack(QXmlStreamWriter* xmlWriter, Activity* act);
    void writeGPXPoint(QXmlStreamWriter* xmlWriter, GpsPoint* point);
    void writeGPXTrkseg(QXmlStreamWriter* xmlWriter, Lap* lap);


public:

    explicit Track(QString fileName,QString name);
    explicit Track();
    virtual ~Track();

    bool readFromXML(QString fileName);
    GpsPoint* parseGpsPoint(QXmlStreamReader& xml);
    QList<GpsPoint*> getGpsPoints();
    inline QString getFileName(){return fileName;}
    inline void setFileName(QString file){fileName=file;}

    inline QString getName(){return name;}
    inline void setName(QString n){name=n;}
    inline void setName(const char* n){name=n;}
    inline int getNumActivities(){return numActivities;}
    inline void setNumActivites(int activities){numActivities=activities;}

    void addActivity(QString sport);
    void addLap();
    Summarized* addPoint(GpsPoint* point);
    QString toString();

    bool saveToXML();

};

#endif // TRACK_H
