#include "utils.h"
#include "constants.h"
#include "track.h"
#include <QFile>
#include <QDebug>
#include <QXmlStreamReader>

uint qHash(const QPoint& p)
{
    return p.x() * 17 ^ p.y();
}

QPointF tileForCoordinate(qreal lat, qreal lng, int zoom)
{
    double zn = static_cast<double>(1 << zoom);
    double tx = (lng + 180.0) / 360.0;

    double ty = (1.0 - log(tan(lat * PI / 180.0) + 1.0 / cos(lat * PI / 180.0)) / PI) / 2.0;

    return QPointF(double(tx*zn), ty * zn);
}

qreal longitudeFromTile(qreal tx, int zoom)
{
    qreal zn = static_cast<qreal>(1 << zoom);
    qreal lat = tx / zn * 360.0 - 180.0;
    return lat;
}

qreal latitudeFromTile(qreal ty, int zoom)
{
    qreal zn = static_cast<qreal>(1 << zoom);
    qreal n = M_PI - 2 * M_PI * ty / zn;
    qreal lng = 180.0 / M_PI * atan(0.5 * (exp(n) - exp(-n)));
    return lng;
}
qreal max(qreal a, qreal b){

    if(a>=b){
        return a;
    }else
        return b;
}

qreal min(qreal a, qreal b){

    if(a<b){
        return a;
    }else
        return b;
}

QString stringDateFromSeconds(double amount){



    int hours = static_cast<int>(amount/3600);
    int minutes = static_cast<int>((static_cast<int>(amount)%3600)/60);
    int seconds = (static_cast<int>(amount)%3600)%60;

    QString dateString=QString("%1h %2m %3s").arg(hours).arg(minutes).arg(seconds);

    //qDebug()<<"amount " << amount << " seconds has been transformated to:" << dateString;
    return dateString;
}

double toRad(double degrees){
    double radians = degrees*PI/180;
    return radians;
}

Track* readFromXML(QString fileName){

    // I need this function to work by now, When I've time I've to make it proper
    // It doesn't work with no valid XML files!!!

    QFile* file= new QFile(fileName);
    bool nameFound=false;

    qDebug() << "going to read file:" << fileName;

    Track* myTrack = new Track();
    myTrack->addActivity("doesn'tmatter");

    if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Error: Cannot read file "
                 << qPrintable(fileName) << ": "
                 << qPrintable(file->errorString());
        return myTrack;
    }

    XMLFileType fileType;

    if(fileName.endsWith(".tcx",Qt::CaseInsensitive)){
        fileType=XMLFile_TCX;
    }else if(fileName.endsWith(".gpx",Qt::CaseInsensitive)){
        fileType=XMLFile_GPX;
    }else{
        qDebug()<<"Error: XML not recognized. " << "nameFile:" << fileName;
    }

    QXmlStreamReader xml(file);

    while (!xml.atEnd() && !xml.hasError()) {

        /* Read next element.*/
        QXmlStreamReader::TokenType token = xml.readNext();
        /* If token is just StartDocument, we'll go to next.*/
        if(token == QXmlStreamReader::StartDocument) {
            continue;
        }
        /* If token is StartElement, we'll see if we can read it.*/
        if(token == QXmlStreamReader::StartElement) {
            // just read Positions, dont care about nothing more...
            // this means, all activities, laps or  track sequences are
            // processed as a single one!

            if(fileType==XMLFile_GPX){
                if(xml.name() == "trkpt") {
                    GpsPoint* point = parseGPXGpsPoint(xml);
                    //qDebug()<<"adding point: " << point->toString();
                    myTrack->addPoint(point);
                }else{
                    continue;
                }
            }else if(fileType==XMLFile_TCX){
                if(xml.name() == "Trackpoint") {
                    GpsPoint* point = parseTCXGpsPoint(xml);
                    //qDebug()<<"adding point: " << point->toString();
                    myTrack->addPoint(point);

                }else{
                    continue;
                }
            }
            if(xml.name()=="name"){
                myTrack->setName(parseStringElement(xml,"name"));
                nameFound=true;
            }
            if(xml.name()=="Activity" && !nameFound){

                myTrack->setName(parseAttribute(xml,"Activity","Sport"));
                nameFound=true;
            }
        }
    }
    /* Error handling. */
    if(xml.hasError()) {
        qDebug() << "Error: in readFromXML "
                 << qPrintable(fileName) << ": "
                 << qPrintable(file->errorString());
    }
    /* Removes any device() or data from the reader
        * and resets its internal state to the initial state. */
    qDebug()<<"clearing xml!";
    xml.clear();
    qDebug() << "going to return myTrack";
    return myTrack;
}

double parseDoubleAttribute(QXmlStreamReader& xml, const char* stringElement, const char* stringAttribute){
    QString sValue = parseAttribute(xml,stringElement,stringAttribute);
    return sValue.toDouble();
}

QString parseAttribute(QXmlStreamReader& xml, const char* stringElement, const char* stringAttribute){

    QString value = QString();
    if(xml.tokenType() == QXmlStreamReader::StartElement && (xml.name() == stringElement)) {
        // Let's get the attributes
        QXmlStreamAttributes attributes = xml.attributes();
        // Let's check that element has the attribute
        if(attributes.hasAttribute(stringAttribute)) {
            value.append(attributes.value(stringAttribute).toString());
        }
    }
    return value;
}

QString parseStringElement(QXmlStreamReader& xml,const char* stringElement){

    // It is suposed the XML is valid!
    QString value = QString();
    if(xml.tokenType() == QXmlStreamReader::StartElement && (xml.name() == stringElement)) {
        /* We've found the element */

        xml.readNext();
        if(xml.tokenType() == QXmlStreamReader::Characters) {

            value.append(xml.text().toString());
        }
    }
    return value;
}

double parseDoubleElement(QXmlStreamReader& xml,const char* stringElement){
    // It is suposed the XML is valid!
    double value=0;
    if(xml.tokenType() == QXmlStreamReader::StartElement && (xml.name() == stringElement)) {
        /* We've found the element */

        xml.readNext();
        if(xml.tokenType() == QXmlStreamReader::Characters) {
            /* Now we can add it to the point.*/
            QString valueString = xml.text().toString();
            value=valueString.toDouble();
        }
    }
    return value;
}

QDateTime parseDateElement(QXmlStreamReader& xml,const char* stringElement){
    // It is suposed the XML is valid!
    QDateTime value;
    if(xml.tokenType() == QXmlStreamReader::StartElement && (xml.name() == stringElement)) {
        /* We've found the element */

        xml.readNext();
        if(xml.tokenType() == QXmlStreamReader::Characters) {
            /* Now we can add it to the point.*/
            QString valueString = xml.text().toString();
            value = QDateTime::fromString(valueString,XML_DATE_FORMAT);
        }
    }
    return value;
}


GpsPoint* parseGPXGpsPoint(QXmlStreamReader& xml){
    GpsPoint* point = new GpsPoint(0,0,0);
    /* Let's check that we're really getting a Trackpoint. */
    if(xml.tokenType() != QXmlStreamReader::StartElement &&
            xml.name() == "trkpt") {
        return point;
    }
    // Let's take latitude and longitude attributes.

    point->setLatitude(parseDoubleAttribute(xml,"trkpt","lat"));
    point->setLongitude(parseDoubleAttribute(xml,"trkpt","lon"));

    xml.readNext();

    while(!(xml.tokenType() == QXmlStreamReader::EndElement &&
            xml.name() == "trkpt")){

        if(xml.tokenType() == QXmlStreamReader::StartElement) {

            if(xml.name() == "ele"){
                //qDebug()<<"found ele element";
                point->setElevation(parseDoubleElement(xml,"ele"));
            }
            if(xml.name() == "time"){
                point->setTime(parseDateElement(xml,"time"));
                //qDebug()<<"found Trackpoint element";
            }
        }
        xml.readNext();
    }
    return point;
}

GpsPoint* parseTCXGpsPoint(QXmlStreamReader& xml){

    GpsPoint* point = new GpsPoint(0,0,0);
    /* Let's check that we're really getting a Trackpoint. */
    if(xml.tokenType() != QXmlStreamReader::StartElement &&
            xml.name() == "Trackpoint") {
        return point;
    }
    xml.readNext();


    while(!(xml.tokenType() == QXmlStreamReader::EndElement &&
            xml.name() == "Trackpoint")){

        if(xml.tokenType() == QXmlStreamReader::StartElement) {

            if(xml.name() == "Time"){
                //qDebug()<<"found Time element";
                point->setTime(parseDateElement(xml,"Time"));
            }
            if(xml.name() == "Position"){
                //qDebug()<<"found Position element";
                xml.readNext();
                while(!(xml.tokenType() == QXmlStreamReader::EndElement &&
                        xml.name() == "Position")){
                    if(xml.tokenType()==QXmlStreamReader::StartElement && xml.name() == "LongitudeDegrees"){
                        //qDebug()<<"found LongitudeDegrees element";
                        point->setLongitude(parseDoubleElement(xml,"LongitudeDegrees"));
                    }if(xml.tokenType()==QXmlStreamReader::StartElement && xml.name() == "LatitudeDegrees"){
                        //qDebug()<<"found LatitudeDegrees element";
                        point->setLatitude(parseDoubleElement(xml,"LatitudeDegrees"));
                    }
                    xml.readNext();
                    //qDebug()<<"element readed: " << xml.name();
                }
            }
            if(xml.tokenType()==QXmlStreamReader::StartElement && xml.name() == "AltitudeMeters"){
                point->setElevation(parseDoubleElement(xml,"AltitudeMeters"));
                //qDebug()<<"found Trackpoint element";
            }
            if(xml.tokenType()==QXmlStreamReader::StartElement && xml.name() == "DistanceMeters"){
                point->setDistancePrev(parseDoubleElement(xml,"DistanceMeters"));
                //qDebug()<<"found Trackpoint element";
            }
        }
        xml.readNext();
    }
    return point;
}

