#include "windowMap.h"

#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QCloseEvent>
#include <QLineEdit>
#include <QAbstractItemView>
#include <QAbstractItemModel>
#include <QFileDialog>
#include <QtNetwork>
#include <QMessageBox>




#include "satellitedialog.h"



WindowMap::WindowMap(QWidget *parent, Settings* settings_p, Log *log)
    : QMainWindow(parent),satelliteInfo(satelliteInfo), satelliteDialog(0),
      log(log),pressed(false),snapped(false), settings(settings_p),track(0),trackToSniff(0),network(false),slippyMap(0),
      waitingForFix(false),validPositions(0),invalidPositions(0),mode(Mode_NewActivityMode),possitionPaint(true),nSession(0),location(0){

    track = new Track();
    trackToSniff = new Track();
    trackToSimulate = QList<GpsPoint*>();

    settingsWindow = new SettingsWindow(this,settings,log);
    settingsWindow->hide();

    loadTracksWindow = new LoadTracksWindow(this,settings,log);
    loadTracksWindow->hide();

    infoWindow = new ActivityInfo(this,settings,log);
    infoWindow->hide();

    //log->debug("tracks created");
    simulateGPS=false;

    autosaveTimer = new QTimer(this);
    sPossitioningTimer = new QTimer(this);
    simulatePossitioningTimer = new QTimer(this);

    zoom=DEFAULT_ZOOM;

    zoomInButton = new QToolButton(this);
    zoomInButton->setIcon(QIcon(":/images/Zoom-In-icon.png"));
    zoomInButton->adjustSize();
    //log->debug(QString("width():%1, zoomInButton.width():%2").arg(width()).arg(zoomInButton->width()));

    zoomOutButton = new QToolButton(this);
    zoomOutButton->setIcon(QIcon(":/images/Zoom-Out-icon.png"));
    zoomOutButton->adjustSize();

    zoomInButton->hide();
    zoomOutButton->hide();

    connect(zoomInButton,SIGNAL(clicked()),this,SLOT(onZoomIn()));
    connect(zoomOutButton,SIGNAL(clicked()),this,SLOT(onZoomOut()));

    connect(autosaveTimer, SIGNAL(timeout()), this, SLOT(autosave()));
    connect(sPossitioningTimer, SIGNAL(timeout()), this, SLOT(activatePossitionPaint()));
    connect(simulatePossitioningTimer, SIGNAL(timeout()), this,SLOT(simulatedPossitionUpdated()));

    sPossitioningTimer->setInterval(1000*TIME_SECONDS_POSSITIONING_PAN);

    // menus

    //log->debug("going to create actions");

    loadTrackAction = new QAction(tr("Load track"),this);
    downloadMapsAction = new QAction(tr("Download maps"),this);
    newActivityAction = new QAction(tr("New activity"),this);
    simulatedActivityAction = new QAction(tr("Activity simulation"),this);
    infoAction = new QAction(tr("Activity info"),this);
    quitAction = new QAction(tr("Quit"), this);

    lapAction = new QAction(tr("New lap"), this);
    settingsAction = new QAction(tr("Settings"), this);
    stopAction = new QAction(tr("Stop"), this);
    startAction = new QAction(tr("Start"), this);
    startAction->setEnabled(false);
    tracksAction = new QAction(tr("Load track"), this);
    satelliteAction = new QAction(tr("Satellite info."), this);
    startNetworkAction = new QAction(tr("Start Networking"),this);
    startNetworkAction->setEnabled(false);

    //log->debug("done, connecting");

    connect(loadTrackAction, SIGNAL(triggered()),SLOT(loadTrack()));
    connect(downloadMapsAction, SIGNAL(triggered()),SLOT(downloadMapTrack()));
    connect(newActivityAction, SIGNAL(triggered()),SLOT(newActivityTrack()));
    connect(simulatedActivityAction, SIGNAL(triggered()), SLOT(newSimulation()));
    connect(stopAction, SIGNAL(triggered()), SLOT(stopActivity()));
    connect(startAction, SIGNAL(triggered()), SLOT(startActivity()));
    connect(quitAction, SIGNAL(triggered()), SLOT(quit()));
    connect(lapAction, SIGNAL(triggered()), SLOT(lapActivity()));
    connect(settingsAction, SIGNAL(triggered()), SLOT(configureSettings()));
    connect(tracksAction, SIGNAL(triggered()), SLOT(tracksActivity()));
    connect(satelliteAction, SIGNAL(triggered()), SLOT(satellite()));
    connect(startNetworkAction,SIGNAL(triggered()), SLOT(networkStart()));
    connect(infoAction, SIGNAL(triggered()), SLOT(info()));


     //log->debug("done!");

     //log->debug("going to show buttons...");
     //log->debug("done?");

    this->setFixedSize(WIDTH_DEFAULT,HEIGHT_DEFAULT);
     //parent->window()->setFixedSize(WIDTH_DEFAULT,HEIGHT_DEFAULT);
}



WindowMap::~WindowMap()
{
    //log->debug("windowMap destructor");
    if(nSession){
        nSession->close();
    }
    //log->debug("session closed");
    if (location){
        location->stopUpdates();
    }
    //log->debug("location closed");

}

void WindowMap::setTrack(QString fileName,QString activity){

    track = new Track(fileName,activity);
    //log->debug(QString("track created!!!:%1").arg(track->toString()));
}

void WindowMap::setMenuLoadTracks(){

    //log->debug("MenuLoadTracks!!!!");

    menuBar()->clear();
#if defined(Q_OS_SYMBIAN) || defined(Q_OS_WINCE_WM) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6)

    menuBar()->addAction(loadTrackAction);
    menuBar()->addAction(downloadMapsAction);
    menuBar()->addAction(newActivityAction);
    menuBar()->addAction(simulatedActivityAction);
    menuBar()->addAction(startNetworkAction);
    menuBar()->addAction(infoAction);
    menuBar()->addAction(quitAction);

#else

    QMenu *menu = menuBar()->addMenu("&Options");
    menu->clear();
    menu->addAction(loadTrackAction);
    menu->addAction(downloadMapsAction);
    menu->addAction(newActivityAction);
    menu->addAction(simulatedActivityAction);
    menu->addAction(startNetworkAction);
    menu->addAction(infoAction);
    menu->addAction(quitAction);

#endif
}

void WindowMap::setMenuActivity(){

    menuBar()->clear();
#if defined(Q_OS_SYMBIAN) || defined(Q_OS_WINCE_WM) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6)
    menuBar()->addAction(lapAction);
    menuBar()->addAction(settingsAction);
    menuBar()->addAction(startAction);
    menuBar()->addAction(stopAction);
    menuBar()->addAction(startNetworkAction);
    menuBar()->addAction(tracksAction);
    menuBar()->addAction(infoAction);
    menuBar()->addAction(quitAction);


    menuBar()->addAction(satelliteAction);

#else
    QMenu *menu = menuBar()->addMenu("&Options");
    menu->clear();
    menu->addAction(lapAction);
    menu->addAction(settingsAction);
    menu->addAction(stopAction);
    menu->addAction(startAction);
    menu->addAction(tracksAction);
    menu->addAction(startNetworkAction);
    menu->addAction(satelliteAction);
    menu->addAction(infoAction);
    menu->addAction(quitAction);


#endif
}

void WindowMap::activatePossitionPaint(){
    //log->debug("activating possitionPaint");
    possitionPaint=true;
    sPossitioningTimer->stop();
}
void WindowMap::desactivatePossitionPaint(){
    //log->debug("desactivating possition paint, and starting timer");
    possitionPaint=false;
    sPossitioningTimer->start();
}

void WindowMap::clearAllMaps(){
    if(slippyMap){
       slippyMap->clearAllMaps();
    }
}


void WindowMap::startServices(){

    startGPS();
    //log->debug("--- startServices ---");
    if(settings->getIsOnline()){
        //log->debug("Is online?" + settings->getIsOnline());
        //log->debug("going to start networking...");
        startNetworking();

    }else{
        //log->debug("going offline");
        startNetworkAction->setEnabled(true);
    }
    startMaps();
}

void WindowMap::showSatelliteInfo(){

    if(!satelliteInfo)
        satelliteInfo = QGeoSatelliteInfoSource::createDefaultSource(this);

    if (satelliteInfo) {
        SatelliteDialog *dialog = new SatelliteDialog(this,
                30,
                SatelliteDialog::ExitOnCancel,
                SatelliteDialog::OrderByPrnNumber,
                SatelliteDialog::ScaleToMaxPossible);

        dialog->connectSources(location, satelliteInfo);

        //location->startUpdates();
        satelliteInfo->startUpdates();

        dialog->exec();

        //satelliteInfo->stopUpdates();

        delete dialog;
        //delete satelliteInfo;
    }

}


void WindowMap::waitForFix() {
    if (waitingForFix)
        return;

    waitingForFix = true;

    satelliteInfo = QGeoSatelliteInfoSource::createDefaultSource(this);

    if (satelliteInfo) {
        satelliteDialog = new SatelliteDialog(this,
                30,
                SatelliteDialog::ExitOnCancel,
                SatelliteDialog::OrderByPrnNumber,
                SatelliteDialog::ScaleToMaxPossible);

        connect(satelliteDialog,SIGNAL(rejected()),satelliteDialog,SLOT(hide()));

        satelliteDialog->connectSources(location, satelliteInfo);

        satelliteInfo->startUpdates();

        satelliteDialog->exec();

        //satelliteInfo->stopUpdates();

        //delete dialog;
        //delete satelliteInfo;
    }

    waitingForFix = false;
}

void WindowMap::stopNetworking(){
    network=false;
    if(nSession)
        nSession->close();
    if(slippyMap)
        slippyMap->network=false;
}

void WindowMap::startNetworking(){

    if(!network && ((mode==Mode_NewActivityMode && settings->getIsOnline()) || (mode==Mode_LoadTracksWindow))){


        // Set Internet Access Point
        QNetworkConfigurationManager manager;
        const bool canStartIAP = (manager.capabilities()
                                  & QNetworkConfigurationManager::CanStartAndStopInterfaces);

#ifdef BEARER_IN_QTNETWORK
        QNetworkConfiguration cfg = manager.defaultConfiguration();
        if (!cfg.isValid() || (!canStartIAP && cfg.state() != QNetworkConfiguration::Active)) {
#else
        QTM_PREPEND_NAMESPACE(QNetworkConfiguration) cfg = manager.defaultConfiguration();
        if (!cfg.isValid() || (!canStartIAP && cfg.state() != QTM_PREPEND_NAMESPACE(QNetworkConfiguration)::Active)) {
#endif

            QMessageBox::information(this, tr("Network"), tr(
                                         "No Access Point found."));
            network=false;
            return;
        }

        nSession = new QNetworkSession(cfg, this);
        nSession->open();
        nSession->waitForOpened(-1);


        connect(nSession, SIGNAL(opened()), this, SLOT(networkSessionOpened()));
        connect(nSession,SIGNAL(error(QNetworkSession::SessionError)),this,SLOT(error(QNetworkSession::SessionError)));


        network=true;
        if(slippyMap){
            slippyMap->network=true;
        }
    }

}

void WindowMap::networkSessionOpened(){

    //log->debug("--- signal recived of sesson opened ---");
    network=true;
    if(slippyMap){
        //log->debug("invalidating map");
        slippyMap->network=true;
        slippyMap->invalidate();

    }

}

void WindowMap::error(QNetworkSession::SessionError error)
{
    //log->debug("--- Error in Networking!!! ---");
    //log->debug(QString("settings is configured?%1").arg(settings->getIsConfigured()));

    if(mode==Mode_NewActivityMode && (!settings->getIsOnline() || !settings->getIsConfigured()))
        return;

    QMessageBox msgBox(this);

    msgBox.setText(tr("Downloading maps requires network access!."));
    msgBox.setInformativeText(tr("Try to connect or cancel to disable maps."));
    msgBox.setIcon(QMessageBox::Information);
    msgBox.setStandardButtons(QMessageBox::Retry | QMessageBox::Cancel);
    msgBox.setDefaultButton(QMessageBox::Retry);

    int ret = msgBox.exec();
    if (ret == QMessageBox::Retry) {
        QTimer::singleShot(0, nSession, SLOT(open()));
    } else if (ret == QMessageBox::Cancel) {
        if(mode==Mode_NewActivityMode){

            settings->setIsOnline(false);
            //log->debug("enabling startNetworking");
            startNetworkAction->setEnabled(true);
            stopNetworking();
        }else{
            QTimer::singleShot(60000, nSession, SLOT(open()));
        }
        network=false;
        slippyMap->network=false;
    }
}

void WindowMap::startMaps(){

    if(!slippyMap){
        //log->debug(QString("network:%1").arg(network));
        slippyMap = new SlippyMap(nSession, this, log, zoom,settings->getMapType());
        connect(slippyMap, SIGNAL(updated(QRect)), SLOT(updateMap(QRect)));
        slippyMap->width = width();
        slippyMap->height = height();
        slippyMap->network=network;
        //log->debug(QString("START MAPS!!! slippy.network:%1").arg(slippyMap->network));
    }
}
void WindowMap::changeUpdateInterval(int millis){

    if(!simulateGPS)
        location->setUpdateInterval(millis);
    else
        simulatePossitioningTimer->setInterval(millis);
}

void WindowMap::changeMapType(MapType map){
    slippyMap->mapType=map;
}

bool WindowMap::checkGPS(){

#if defined(Q_OS_SYMBIAN) || defined(Q_OS_WINCE_WM) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6)
    location = QGeoPositionInfoSource::createDefaultSource(this);

#else
    //With maemo simulator doesn't work, I've to change this.
    location=0;
#endif

    if (!location) {
        if(!simulateGPS)
            QMessageBox::information(this, tr("No GPS detected"),
                                     tr("No GPS support detected, going to simulate mode"));
        simulateGPS=true;
    }

}

void WindowMap::startGPS(){


   if(location && !simulateGPS){
        location->setUpdateInterval(settings->getGpsInterval());
        connect(location,SIGNAL(positionUpdated(QGeoPositionInfo)),this,SLOT(positionUpdated(QGeoPositionInfo)));
        if(!settings->getIsOnline())
            location->setPreferredPositioningMethods(QGeoPositionInfoSource::SatellitePositioningMethods);

        location->startUpdates();
        waitForFix();
        //log->debug("setting signal updateTimeout...");
        connect(location, SIGNAL(updateTimeout()), this, SLOT(waitForFix()));
        //log->debug("start positioning...");
    }
    startPositioning();
}

void WindowMap::setTopLeft(qreal lat, qreal lng){

    //log->debug("----setTopLeft----");
    QPointF ct = tileForCoordinate(lat, lng, zoom);

    qreal tx = ct.x()-static_cast<double>(width())/(2.0*static_cast<double>(tdim));
    qreal ty = ct.y()-static_cast<double>(height())/(2.0*static_cast<double>(tdim));
    topLeft=QPointF(tx,ty);
}

void WindowMap::setCenter(qreal lat, qreal lng) {

    //log->debug("--- set Center function ---!");

    if (!slippyMap) {
        log->err("slippyMap not created!");
        firstLat = lat;
        firstLong = lng;
        return;
    }
    slippyMap->latitude = lat;
    slippyMap->longitude = lng;

    setTopLeft(lat, lng);
    slippyMap->invalidate();
}

void WindowMap::updateMap(const QRect &r) {
    update(r);
}

void WindowMap::closeEvent(QCloseEvent *event){

    //log->debug("---- WindowMap quit, going to show parent and hide this ----");
    //log->debug(QString("mode:==Mode_NewActivityMode?:%1, mode:%2").arg(mode==Mode_NewActivityMode).arg(mode));
    if(mode==Mode_NewActivityMode && !simulateGPS){
        QMessageBox msgBox(this);

        msgBox.setText(tr("Do you want to close activity?, it will be saved."));
        msgBox.setInformativeText(tr("Closing Activity"));
        msgBox.setIcon(QMessageBox::Information);
        msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
        msgBox.setDefaultButton(QMessageBox::Cancel);
        int ret = msgBox.exec();

        if (ret == QMessageBox::Cancel){
            event->ignore();
        }else{
            stopPositioning();
            autosave();
            settings->setIsConfigured(false);
            //log->debug("SETTING CONFIGURE SETTINGS TO FALSE");
            this->parentWidget()->show();
            this->hide();
            //log->debug("done");
        }
    }else{

        if(mode==Mode_NewActivityMode && simulateGPS){
            settings->setIsConfigured(false);
            //log->debug("exit from simulating GPS");
            //log->debug("setting isconfigured to false and stopping timer");
            simulatePossitioningTimer->stop();
            simulateGPS=false;
        }

        this->parentWidget()->show();
        this->hide();
        //log->debug("done");
    }
}

 void WindowMap::showEvent(QShowEvent *){


     if(mode==Mode_NewActivityMode && !settings->getIsConfigured()){
         checkGPS();
         settingsWindow->show();
         //setWindowTitle(settings->getActivity());
     }else if(mode==Mode_LoadTracksWindow && !settings->getIsConfigured()){
         setWindowTitle(QString("Load Tracks ->").append(trackToSniff->getName()));
         loadTracksWindow->show();
     }else{
        fixZoomButtons();
     }
 }

 void WindowMap::networkStart(){
     startNetworkAction->setEnabled(false);
     QTimer::singleShot(0, nSession, SLOT(open()));
 }

 void WindowMap::configureSettings(){
    settingsWindow->show();
 }

 void WindowMap::tracksActivity(){
     //mode=Mode_LoadTracksWindow;
     possitionPaint=false;
     //setMenuLoadTracks();
     loadTracksWindow->show();
 }

 void WindowMap::loadTrack(){
     loadTracksWindow->show();
 }
 void WindowMap::downloadMapTrack(){}
 void WindowMap::newActivityTrack(){
     mode=Mode_NewActivityMode;
     settings->setIsConfigured(false);
     setMenuActivity();
     settingsWindow->show();
 }

 void WindowMap::newSimulation(){
     mode=Mode_NewActivityMode;
     simulateGPS=true;
     settings->setIsConfigured(false);
     setMenuActivity();
     settingsWindow->show();
 }

 void WindowMap::info(){
    infoWindow->show();
 }

 void WindowMap::startActivity(){

     stopAction->setEnabled(true);
     startAction->setEnabled(false);
     startPositioning();
     //log->debug("stopStart enabled");
}
void WindowMap::stopActivity(){

    stopAction->setEnabled(false);
    startAction->setEnabled(true);

    stopPositioning();

}

/*
void WindowMap::settings(){
    settingsWindow->show();
}
*/

void WindowMap::lapActivity(){
    if(track)
        track->addLap();
}

void WindowMap::quit(){
    QCloseEvent *event = new QCloseEvent();
    emit closeEvent(event);
}
/*
void WindowMap::tracks(){

}
*/
void WindowMap::satellite(){

#if defined(Q_OS_SYMBIAN) || defined(Q_OS_WINCE_WM) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6)
    showSatelliteInfo();
#endif
}

void WindowMap::stopPositioning() {
   if(simulateGPS){
    simulatePossitioningTimer->stop();
   }else{
    if (location)
        location->stopUpdates();
   }
}

void WindowMap::startPositioning() {
    if(simulateGPS){
        simulatePossitioningTimer->start();
    }else{
        if (location)
            location->startUpdates();
    }
}

void WindowMap::onZoomOut(){

    int minzoom=0;
    if(settings->getMapType()==MapTypeICC)
        minzoom=MIN_ICC_ZOOM;

    if (zoom > minzoom) {
        --(zoom);
        zoomOutButton->setEnabled(zoom > minzoom);
        zoomInButton->setEnabled(true);

        if (!slippyMap)
            return;
        slippyMap->zoom=zoom;

        setTopLeft(slippyMap->getLatitude(), slippyMap->getLongitude());
        slippyMap->invalidate();
    }

}

void WindowMap::onZoomIn(){
    if (zoom < MAX_ZOOM - 1) {
        ++(zoom);
        zoomInButton->setEnabled(zoom < MAX_ZOOM - 1);
        zoomOutButton->setEnabled(true);

        if (!slippyMap)
            return;
        slippyMap->zoom=zoom;
        setTopLeft(slippyMap->getLatitude(), slippyMap->getLongitude());
        slippyMap->invalidate();
    }

}

void WindowMap::updateSummary(){
    emit(infoWindow->updateSummary());
}

void WindowMap::simulatedPossitionUpdated(){
    log->info("simulated possition!!!");
    //log->info(QString("num points to simulate:%1").arg(trackToSimulate.size()));
    GpsPoint* point= new GpsPoint(0,0,0);


    if(trackToSimulate.size()>0){

        point = trackToSimulate.first();

        log->debug(QString("point to simulate:%1").arg(point->toString()));
        track->addPoint(point);
        log->debug("going to emit updateSummary!!!");
        emit(infoWindow->updateSummary());
        log->debug("point added to track and updated");
        //log->debug(QString("track num points:%1").arg(track->getNumPoints()));

        trackToSimulate.removeFirst();
        validPositions++;
        //log->info(QString("num points to simulate:%1").arg(trackToSimulate.size()));

        //log->debug(QString("Possition paint???????????%1").arg(possitionPaint));
        if(possitionPaint)
            setCenter(point->getLatitude(),point->getLongitude());
    }



}

void WindowMap::positionUpdated(const QGeoPositionInfo &pos){

    //log->info("ACTIVITYMAP-POSSITIONUPDATED!!!");
    //setCenter(pos.coordinate().latitude(), pos.coordinate().longitude());

    if(pos.isValid() &&
            pos.attribute(QGeoPositionInfo::HorizontalAccuracy)<=settings->getHorizontalAccuracy() &&
            pos.attribute(QGeoPositionInfo::VerticalAccuracy)<=settings->getVerticalAccuracy()){

        GpsPoint* point = new GpsPoint(pos.coordinate().latitude(),pos.coordinate().longitude(),pos.coordinate().altitude(),pos.timestamp(),
                                       pos.attribute(QGeoPositionInfo::GroundSpeed),pos.attribute(QGeoPositionInfo::Direction),
                                       pos.attribute(QGeoPositionInfo::MagneticVariation),
                                       pos.attribute(QGeoPositionInfo::HorizontalAccuracy),
                                       pos.attribute(QGeoPositionInfo::VerticalAccuracy));


       // log->debug(point->toString());
        //log->debug("going to add point...");
        track->addPoint(point);
        emit(infoWindow->updateSummary());
        validPositions++;

        if(possitionPaint)
            setCenter(point->getLatitude(),point->getLongitude());

        if(satelliteDialog)
            satelliteDialog->hide();


    }else{
        log->debug("position not updated:");

        log->debug("Horizontal accuracy (meters):" + QString::number(pos.attribute(QGeoPositionInfo::HorizontalAccuracy)));
        log->debug("Vertical accuracy (meters):" + QString::number(pos.attribute(QGeoPositionInfo::VerticalAccuracy)));
        invalidPositions++;
    }

}


void WindowMap::autosave(){

    if(track!=0 && track->getNumPoints()>0 && !simulateGPS){
        log->debug("going to save the TCX file...");
        track->saveToXML();
        log->debug("done!");
    }
    log->info("done...");
}

void WindowMap::setOrientation(ScreenOrientation orientation)
{
#if defined(Q_OS_SYMBIAN)
    // If the version of Qt on the device is < 4.7.2, that attribute won't work
    if (orientation != ScreenOrientationAuto) {
        const QStringList v = QString::fromAscii(qVersion()).split(QLatin1Char('.'));
        if (v.count() == 3 && (v.at(0).toInt() << 16 | v.at(1).toInt() << 8 | v.at(2).toInt()) < 0x040702) {
            qWarning("Screen orientation locking only supported with Qt 4.7.2 and above");
            return;
        }
    }
#endif // Q_OS_SYMBIAN

    Qt::WidgetAttribute attribute;
    switch (orientation) {
#if QT_VERSION < 0x040702
    // Qt < 4.7.2 does not yet have the Qt::WA_*Orientation attributes
    case ScreenOrientationLockPortrait:
        attribute = static_cast<Qt::WidgetAttribute>(128);
        break;
    case ScreenOrientationLockLandscape:
        attribute = static_cast<Qt::WidgetAttribute>(129);
        break;
    default:
    case ScreenOrientationAuto:
        attribute = static_cast<Qt::WidgetAttribute>(130);
        break;
#else // QT_VERSION < 0x040702
    case ScreenOrientationLockPortrait:
        attribute = Qt::WA_LockPortraitOrientation;
        break;
    case ScreenOrientationLockLandscape:
        attribute = Qt::WA_LockLandscapeOrientation;
        break;
    default:
    case ScreenOrientationAuto:
        attribute = Qt::WA_AutoOrientation;
        break;
#endif // QT_VERSION < 0x040702
    };
    setAttribute(attribute, true);
}

void WindowMap::showExpanded()
{
#ifdef Q_OS_SYMBIAN
    showFullScreen();
#elif defined(Q_WS_MAEMO_5)
    showMaximized();
#else
    show();
#endif
}

void WindowMap::paintEvent(QPaintEvent *event) {

    fixZoomButtons();

    log->debug("---- paint Event ----!");

    QPainter p;
    p.begin(this);

    p.setPen(Qt::black);
#if defined(Q_OS_SYMBIAN)
    QFont font = p.font();
    font.setPixelSize(13);
    p.setFont(font);
#endif

    if (!slippyMap){
        log->debug("slippyMap not created!");
        return;
    }

    slippyMap->render(&p, event->rect());

    //log->info("going to draw tracking route...");
    drawTrack(trackToSniff,&p, settings->getTrackToSniffColor());

    if(mode==Mode_NewActivityMode){
        //log->info("going to draw actual route...");
        //log->debug(QString("num points track:%1").arg(track->getNumPoints()));
        //log->debug("entering to drawTrack");
        drawTrack(track,&p,settings->getTrackColor(),true);
        //log->info("done");
    }

    //log->info("done");
    //p.drawText(rect(), Qt::AlignBottom | Qt::TextWordWrap,"OpenStreetMap maps");
    //log->debug("ending Paint event...");
    p.end();
    //log->debug("done... and now?");

}
void WindowMap::drawFullTrackMaps(Track* track_p){

    QList<GpsPoint*> gpsPoints = track_p->getGpsPoints();
    //log->debug(QString("number of points to draw:%1").arg(gpsPoints.size()));

    int zoomToTrack=18;
    bool found=false;
    bool changeZoom;
    bool firstZoom;
    int border=0;

    double min_x;
    double max_x;
    double min_y;
    double max_y;

    while(zoomToTrack>0 && !found){
        changeZoom=false;
        firstZoom=true;
        //log->debug(QString("zoomToTrack:%1").arg(zoomToTrack));
        for (int i = 0;!changeZoom && i < gpsPoints.size(); ++i) {
            GpsPoint* point = gpsPoints.at(i);
            QPointF ct = tileForCoordinate(point->getLatitude(), point->getLongitude(), zoomToTrack);

            //log->debug(QString("point %1: x=%2,y=%3").arg(i).arg(ct.x()).arg(ct.y()));
            if(firstZoom){
                min_x=ct.x();
                max_x=ct.x();
                min_y=ct.y();
                max_y=ct.y();
                firstZoom=false;
            }else{
                min_x = min(min_x,ct.x());
                max_x = max(max_x,ct.x());
                min_y = min(min_y,ct.y());
                max_y = max(max_y,ct.y());
            }
            //log->debug(QString("min_x:%1, max_x:%2, min_y:%3, max_y:%4").arg(min_x).arg(max_x).arg(min_y).arg(max_y));

            // if point is outside screen change zoom
            double d_x=(max_x-min_x)*tdim;
            double d_y=(max_y-min_y)*tdim;
            //log->debug(QString("x distance:%1, y distance:%2").arg(d_x).arg(d_y));
            //log->debug(QString("width-border*2=%1, height-border*2=%2").arg(width()-border*2).arg(height()-border*2));
            if(d_x>(width()-border*2) || d_y>(height()-border*2)){
                //log->debug("point outside screen, changezoom");
                changeZoom=true;
                zoomToTrack--;
            }
        }
        if(!changeZoom){
            found=true;
            //log->debug(QString("all points are inside screen, zoom found: %1").arg(zoomToTrack));
        }
    }

    double med_x=double(min_x+static_cast<double>(width())/(2.0*static_cast<double>(tdim)));
    double med_y=double(min_y+static_cast<double>(height())/(2.0*static_cast<double>(tdim)));
    double lng=longitudeFromTile(med_x,zoomToTrack);
    double lat=latitudeFromTile(med_y,zoomToTrack);

    zoom=zoomToTrack;
    slippyMap->zoom=zoom;
    //log->debug(QString("changing zoom to:%1").arg(zoom));
    //log->debug(QString("centering with x_med:%1,y_med:%2 ").arg(med_x).arg(med_y));
    //log->debug(QString("centering with lat:%1,lng:%2 ").arg(lat).arg(lng));

    slippyMap->latitude = lat;
    slippyMap->longitude = lng;

    //log->debug(QString("minx:%1, miny:%2").arg(min_x).arg(min_y));
    topLeft=QPointF(min_x,min_y);
    slippyMap->invalidate(med_x,med_y);


}
bool WindowMap::drawTrack(Track* track_p, QPainter* p, QString color_s){
    drawTrack(track_p,p,color_s,false);
}

bool WindowMap::drawTrack(Track* track_p, QPainter* p, QString color_s,bool lastPoint){

    if (track_p!=NULL && track_p->getNumPoints()<=0){
        log->debug("no points to draw...");
        return false;
    }
    QColor color = QColor(color_s);
    p->setBrush(QBrush(color));
    QPen pen(color, 5, Qt::SolidLine);

    p->setPen(pen);
    double border=0;

    QPointF previous=QPointF(0,0);

    log->debug("going to get points...");
    //log->debug(QString("num:%1").arg(track_p->getNumPoints()));
    QList<GpsPoint*> points = track_p->getGpsPoints();
    log->debug(QString("drawing with zoom:%1").arg(zoom));
    //log->debug(QString("TopLeft x:%1, y:%2").arg(topLeft.x()).arg(topLeft.y()));
    for(int i = 0; i < points.size(); ++i){

        GpsPoint* point = points.at(i);
        QPointF ct = tileForCoordinate(point->getLatitude(), point->getLongitude(), zoom);

        //log->debug(QString("point %1: x=%2,y=%3").arg(i).arg(ct.x()).arg(ct.y()));
        //log->debug(QString("topLeft.x:%1, topLeft.y:%2").arg(topLeft.x()).arg(topLeft.y()));
        double x_resized = (ct.x() - topLeft.x())*tdim + border;
        double y_resized = (ct.y() - topLeft.y())*tdim + border;
        QPointF resized = QPoint(x_resized,y_resized);
        //log->debug(QString("point resized: x=%1,y=%2").arg(resized.x()).arg(resized.y()));
        if(!previous.isNull() && previous!=ct &&
                (resized.x()>= 0 && resized.x()<=width() &&  resized.y()>=0 && resized.y() <= height())){
             //log->debug(QString("drawing line from: %1,%2 to %3,%4").arg(previous.x()).arg(previous.y()).arg(resized.x()).arg(resized.y()));
             p->drawLine(previous,resized);
        }
        previous=resized;
    }

    if(lastPoint){
        p->setBrush(QBrush(QColor(color_s)));
        p->drawEllipse(previous,10,10);
        p->setBrush(QBrush(Qt::black));
        p->drawEllipse(previous,7,7);
    }
}



void WindowMap::mousePressEvent(QMouseEvent *event) {
    if (!slippyMap)
        return;

    if (event->buttons() != Qt::LeftButton)
        return;
    pressed = snapped = true;
    pressPos = dragPos = event->pos();

    possitionPaint=false;

    //tapTimer.stop();
    //tapTimer.start(HOLD_TIME, this);
}

void WindowMap::mouseMoveEvent(QMouseEvent *event) {
    if (!slippyMap)
        return;

    if (!event->buttons())
        return;

    //possitionPaint=false;

    if (!pressed || !snapped) {
        QPoint delta = event->pos() - pressPos;
        pressPos = event->pos();
        QPointF dx = QPointF(delta);

        //log->debug(QString("Changing topleft, from %1,%2").arg(topLeft.x()).arg(topLeft.y()));
        //log->debug(QString("distance changed: x=%1,y=%2").arg(dx.x()).arg(dx.y()));
        topLeft = topLeft -dx/tdim;
        //log->debug(QString("to %1,%2").arg(topLeft.x()).arg(topLeft.y()));
        //topLeft = QPoint(delta.x()-width()/2,delta.y()-height()/2);
        slippyMap->pan(delta);

        return;
    } else {
        const int threshold = 10;
        QPoint delta = event->pos() - pressPos;
        if (snapped) {
            snapped &= delta.x() < threshold;
            snapped &= delta.y() < threshold;
            snapped &= delta.x() > -threshold;
            snapped &= delta.y() > -threshold;
        }

    }

}

void WindowMap::mouseReleaseEvent(QMouseEvent *) {
    if (!slippyMap)
        return;
    desactivatePossitionPaint();
    update();

}

void WindowMap::fixZoomButtons(){

    if(settings->getIsConfigured()){
            int x_in=(width()-zoomInButton->width());
            int y_in=(height()/2-zoomInButton->height()/2);
            //log->debug(QString("ZoomIn possition: x=%1,y=%2").arg(x_in).arg(y_in));

            int x_out=0;
            int y_out=(height()/2-zoomOutButton->height()/2);
            //log->debug(QString("ZoomOut possition: x=%1,y=%2").arg(x_out).arg(y_out));

            zoomInButton->move(x_in,y_in);
            zoomOutButton->move(x_out,y_out);
            zoomInButton->show();
            zoomOutButton->show();

        }
}

void WindowMap::resizeEvent(QResizeEvent * /* event */)
{

    fixZoomButtons();

    /*
    int x = width() - (zoomInButton->width()
                       + zoomOutButton->width() + 10);
    zoomInButton->move(x, 5);
    zoomOutButton->move(x + zoomInButton->width() + 5, 5);

    */
}


/* QString activity;
    int gpsInterval;
    QString mapType;
    bool isOnline;
    QFile gpxFile;
    */






