#ifndef ACTIVITYMAP_H
#define ACTIVITYMAP_H

#include <QMainWindow>
#include <QDir>
#include <QNetworkSession>
#include <QTimer>
#include <QToolButton>


//QtMobility API headers
#include <qmobilityglobal.h>
#include <qgeosatelliteinfosource.h>
#include <qnmeapositioninfosource.h>
#include <qgeopositioninfo.h>
#include <qnetworkconfigmanager.h>
#include <qnetworksession.h>

#include "log.h"
#include "track.h"
#include "slippyMap.h"
#include "loadTracksWindow.h"
#include "settingsWindow.h"
#include "activityinfo.h"

class Settings;
class ConnectivityHelper;
class SatelliteDialog;


// Use the QtMobility namespace
QTM_USE_NAMESPACE


class WindowMap : public QMainWindow
{
    Q_OBJECT

private:

    int zoom;
    // Atributes
    qreal firstLat;
    qreal firstLong;

    Mode mode;

    bool network;

    bool isConfigured;

    bool pressed;
    bool snapped;
    bool possitionPaint;


    QPoint pressPos;
    QPoint dragPos;

    QPointF topLeft;
    QGeoPositionInfoSource *location;

    SlippyMap* slippyMap;

    QNetworkSession *nSession;
    ConnectivityHelper *nConHelp;
    QString networkError;


    //Menus

    QAction *startNetworkAction;
    QAction *loadTrackAction;
    QAction *downloadMapsAction;
    QAction *newActivityAction;
    QAction *simulatedActivityAction;
    QAction *quitAction;
    QAction *lapAction;
    QAction *settingsAction;
    QAction *stopAction;
    QAction *startAction;
    QAction *tracksAction;
    QAction *satelliteAction;
    QAction *infoAction;

    // Dialogs
    LoadTracksWindow *loadTracksWindow;
    SettingsWindow *settingsWindow;
    ActivityInfo *infoWindow;

    QGeoSatelliteInfoSource *satelliteInfo;
    SatelliteDialog *satelliteDialog;


    Log *log;

    Track *track;
    Track *trackToSniff;

    QList<GpsPoint*> trackToSimulate;

    bool waitingForFix;
    int validPositions;
    int invalidPositions;

    QTimer *autosaveTimer;
    QTimer *sPossitioningTimer;
    QTimer *simulatePossitioningTimer;

    bool simulateGPS;

    // events
    void closeEvent(QCloseEvent *event);
    void showEvent(QShowEvent *);
    void paintEvent(QPaintEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void resizeEvent(QResizeEvent * );

public:

    QToolButton *zoomInButton;
    QToolButton *zoomOutButton;

    Settings *settings;

    enum ScreenOrientation {
        ScreenOrientationLockPortrait,
        ScreenOrientationLockLandscape,
        ScreenOrientationAuto
    };

    explicit WindowMap(QWidget *parent = 0,Settings* settings=0, Log *log=0);
    virtual ~WindowMap();

    void fixZoomButtons();
    void updateSummary();
    void clearAllMaps();
    void setMenuActivity();
    void setMenuLoadTracks();
    void setTopLeft(qreal lat, qreal lng);
    bool drawTrack(Track* track_p, QPainter* p, QString color, bool boolLast);
    bool drawTrack(Track* track_p, QPainter* p, QString color);
    void drawFullTrackMaps(Track* track_p);

    void changeMapType(MapType map);
    void changeUpdateInterval(int millis);
    bool checkGPS();
    void startGPS();
    void startMaps();

    void showSatelliteInfo();
    void startNetworking();
    void stopNetworking();
    void setOrientation(ScreenOrientation orientation);
    void showExpanded();
    void startServices();
    inline Track* getTrack(){return track;}
    inline Track* getTrackToSniff(){return trackToSniff;}
    inline void setTrackToSimulate(QList<GpsPoint*> toDo){trackToSimulate=toDo;}
    inline bool hasTrackToSimulate(){(trackToSimulate.size()>0);}
    inline void startAutosaveTimer(int milliseconds){autosaveTimer->start(milliseconds);}
    inline void startSimulatePossitioning(int milliseconds){simulatePossitioningTimer->start(milliseconds);}
    inline void startSimulatePossitioning(){simulatePossitioningTimer->start();}
    inline void startSPositioningTimer(){sPossitioningTimer->start();}
    inline void saveXml(){autosave();}
    inline void setMode(Mode m){mode=m;}
    inline Mode getMode(){return mode;}
    inline void setTrackToSniff(Track* t){trackToSniff=t;}
    inline bool simulatingGPS(){return simulateGPS;}
    inline void setSimulateGPS(bool value){simulateGPS=value;}
    void setTrack(QString filename,QString activity);
    void setCenter(qreal lat, qreal lng);


    public slots:
    void desactivatePossitionPaint();
    void activatePossitionPaint();
    void simulatedPossitionUpdated();

    private slots:
    void networkSessionOpened();
    void error(QNetworkSession::SessionError error);
    void stopActivity();
    void startActivity();
    void configureSettings();
    void lapActivity();
    void tracksActivity();
    void newSimulation();
    void info();

    void loadTrack();
    void downloadMapTrack();
    void newActivityTrack();
    void networkStart();


    void quit();
    void satellite();

    void onZoomIn();
    void onZoomOut();
    void startPositioning();
    void stopPositioning();


    //void networkSessionOpened();
    void waitForFix();
    void updateMap(const QRect &r);


    void positionUpdated(const QGeoPositionInfo &info);
    void autosave();

};

#endif // ACTIVITYMAP_H
